/*
   Unix SMB/Netbios implementation.
   Version 1.9.
   Copyright (C) Andrew Tridgell 1994-1997

   This file is part of the port to OpenVMS
   Directory routines.
   Copyright (C) Eckart Meyer 1996-1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*	dir.c
 *				5-Aug-1995	IfN/Mey
 *
 *	Derived from a module from P Kay's UNIXSHR.
 *
 *	Modified by Eckart Meyer, meyer@ifn.ing.tu-bs.de
 *+
 * opendir, readdir, seekdir and closedir.
 *-
 */

/* 04-Jul-1998	J. Malmberg	For GCC use the GCC headers */
/* 30-Oct-1998	J. Malmberg	Fix up for using long file name modules */
#ifdef __GNUC__
#include "conf.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#ifndef __GNUC__
#include <lib$routines.h>
#endif
#include <ssdef.h>
#include <rmsdef.h>
#include <errno.h>
#include "longfile_vms.h"

#ifdef __GNUC__
unsigned long LIB$MOVTC
   (const struct dsc$descriptor_s * src,
    const struct dsc$descriptor_s * fill,
    const struct dsc$descriptor_s * trans,
    struct dsc$descriptor_s * dst);

unsigned long LIB$FIND_FILE
   (const struct dsc$descriptor_s * filespec,
    struct dsc$descriptor_s * res_filespec,
    unsigned long * context,
    const struct dsc$descriptor_s * def_filespec,
    const struct dsc$descriptor_s * rel_filespec,
    unsigned long * status_value,
    const unsigned long flags);

unsigned long LIB$FIND_FILE_END(const unsigned long * context);
#endif

#ifndef __UNISTD_LOADED
char *getcwd (char *__buffer, unsigned int __size);
#endif

 /* Fake out the VMS linker for a global symbol */
/*---------------------------------------------*/
char * LIB$AB_LOWERCASE();

#ifndef __GNUC__
struct _dirdesc {
 int dd_fd;
 long dd_loc;
 long dd_size;
 long dd_bbase;
 long dd_entno;
 long dd_bsize;
 char *dd_buf;
};
#endif

/*
 * readdir() should return files relative to the path specified by
 * opendir().

 * 30-Oct-1998	J. Malmberg	Q & D hack.  opendir() needs to allocate a
				memory structure to contain the setup
				information.  This would contain the
				original path specification.

				This memory structure should also contain
				the context variable dd_loc that the readdir
				would use.

				We cheat here by making sure that the
				path is in VMS format, and then storing
				it in the static storage.

*/

static DIR directory;
static struct dirent dp;
const static char *wild = "*.*;0";
static char path[256];
static char resfile[256];
/* static char save_path[256]; */
unsigned long dd_loc;

#if 1	/* for use with samba's debugging */	/* SAMBA */
extern int DEBUGLEVEL;
#define DEBUG(level,body) ((DEBUGLEVEL>=(level))?(Debug1 body):0)
int Debug1();
#else
int DEBUGLEVEL = 8;
#define DEBUG(level,body) ((DEBUGLEVEL>=(level))?(printf body):0)
#endif


DIR *vms_opendir(const char * filename)
{
    DEBUG (4,("opendir: \"%s\"\n",filename));
    if (strlen(filename) > 250)
    {
	errno = ENAMETOOLONG;
	return(NULL);
    }

/*    longfile_to_vms(path, filename, LONGFILE_M_DEEPDIR); */
    strcpy(path,filename);

    DEBUG(8,("   path: \"%s\"\n",path));
/*    directory.dd_loc = 0; */
    dd_loc = 0;

    return(&directory);
}

struct dirent *vms_readdir(DIR * dirp)
{
unsigned long call_stat;
struct dsc$descriptor_s filedesc;
struct dsc$descriptor_s res_filedesc;
struct dsc$descriptor_s pathdesc;

    filedesc.dsc$w_length = strlen(wild);
    (const char *)filedesc.dsc$a_pointer = wild;
    filedesc.dsc$b_dtype = DSC$K_DTYPE_T;
    filedesc.dsc$b_class = DSC$K_CLASS_S;

    res_filedesc.dsc$w_length = 256;
    res_filedesc.dsc$a_pointer = resfile;
    res_filedesc.dsc$b_dtype = DSC$K_DTYPE_T;
    res_filedesc.dsc$b_class = DSC$K_CLASS_S;

    pathdesc.dsc$w_length = strlen(path);
    pathdesc.dsc$a_pointer = path;
    pathdesc.dsc$b_dtype = DSC$K_DTYPE_T;
    pathdesc.dsc$b_class = DSC$K_CLASS_S;

    call_stat = LIB$FIND_FILE
	(&filedesc,
	 &res_filedesc,
/*	 &directory.dd_loc, */
	 &dd_loc,
	 &pathdesc, NULL, NULL, 0);

    if (call_stat == SS$_NORMAL || call_stat == RMS$_NORMAL)
    {
    struct dsc$descriptor_s trans_desc;
    struct dsc$descriptor_s fill_desc;
    char * tptr1;
    char * tptr2;
    char * tptr3;

	trans_desc.dsc$w_length = 256;
	trans_desc.dsc$a_pointer = (char *)LIB$AB_LOWERCASE;
	trans_desc.dsc$b_dtype = DSC$K_DTYPE_T;
	trans_desc.dsc$b_class = DSC$K_CLASS_S;

	fill_desc.dsc$w_length = 1;
	fill_desc.dsc$a_pointer = " ";
	fill_desc.dsc$b_dtype = DSC$K_DTYPE_T;
	fill_desc.dsc$b_class = DSC$K_CLASS_S;

	resfile[res_filedesc.dsc$w_length] = '\0';
        call_stat = LIB$MOVTC
	    (&res_filedesc,
	     &fill_desc,
	     &trans_desc,
	     &res_filedesc);

	tptr1 = strrchr(resfile, ']');
	if (tptr1 == NULL)
	    tptr1 = strrchr(resfile, '>');

	if (tptr1 != NULL)
	    tptr1++;
	tptr2 = strchr(tptr1, '.');

	tptr3 = strchr(tptr2, ';');
	if (tptr3 == NULL)
	    tptr3 = strrchr(tptr1,'.');
	if (tptr3 != NULL)
	    *tptr3 = '\0';

	if (tptr2 != NULL)
	{
	    if (*tptr2 != '\0')
	    {
	    int tcmp;

		DEBUG (8,("readdir: extension: \"%s\"\n",tptr2));

		 /* Samba - remove .dir */
		/*---------------------*/
		tcmp = strcmp(tptr2,".dir");
		if (tcmp == 0)
		    *tptr2 = '\0';
	    }
	}


	strcpy(dp.d_name, tptr1);
	DEBUG (3,("readdir: returns \"%s\"\n",dp.d_name));
	return(&dp);
    }
    if (call_stat == RMS$_NMF)
	return(NULL);	/* don't touch errno if no more files */

    errno = ENOENT;
    if (call_stat == SS$_NOPRIV)
	errno = EACCES;
    return(NULL);
}

int vms_closedir(dirp)
DIR *dirp;
{
/*    lib$find_file_end(&directory.dd_loc); */
    LIB$FIND_FILE_END(&dd_loc);
    return(0);
}

void vms_rewinddir(dirp)
DIR *dirp;
{
/*    lib$find_file_end(&directory.dd_loc); */
/*    directory.dd_loc = 0; */
    LIB$FIND_FILE_END(&dd_loc);
    dd_loc = 0;

/*    return(0); */
}
