/*
 * Unix SMB/Netbios implementation. Version 1.9. SMB parameters and setup
 * Copyright (C) Andrew Tridgell 1992-1998 Modified by Jeremy Allison 1995.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675
 * Mass Ave, Cambridge, MA 02139, USA.



 12-JUN-1998, David Mathog, Biology Division, Caltech

    First hack at this.  Password file uses a simple fixed length binary
    record organization that is search sequentially.  These are laid out like:

    username         32 bytes (ascii text, padded right with nulls)
    uic               4 bytes (stored as Integer)
    acct_ctrl         4 byte lower 16 correspond to acct_ctrl in SMB.H.
                        '10000'x is disabled lanman passwd
                        '20000'x is disabled wnt_passwd
                        (none of this 'X' stuff, as in Unix, since there is 
                        no text form of the password file)
    lanman_password  16 bytes
    wnt_password     16 bytes

    While it is possible to manipulate this kind of file in C, I'm a bit
    more adept at doing it in Fortran, so, the C routines are reduced to
    wrappers around the Fortran.  Not portable, but since this is a platform
    adaptation anyway.  This should be an indexed file anyway.

*/

#include "includes.h"
/* prototypes for fortran pieces
   watch out for "name"!!!  It must be 32 characters long and must
   match internal representation of username (leftpadded with spaces) */

/* global variables used when talking to fortran */

int fsmb_status;
int fsmb_unum=10;

#define ACB_NOLANMANPWD  0x10000  /* 1 = no lanman password has been defined */
#define ACB_NOWNTPWD     0x20000  /* 1 = no NT password has been defined */

struct fsmb_type {
	char username[32];
	int  uic;
	int  acct_ctrl;
	char lanman_password[16];
	char wnt_password[16];
};

void fsmb_openfile(char * filename, int * unum, int * status);
void fsmb_closefile(int * unum, int * status);
void fsmb_find_by_name(int * unum, char * name, struct fsmb_type * data, int * status);
void fsmb_find_by_uic(int * unum, int * uic, struct fsmb_type * data, int * status);
void fsmb_append_entry(int * unum, struct fsmb_type * data, int * status);
void fsmb_modify_last_entry(int * unum, struct fsmb_type * data, int * status);
void fsmb_delete_last_entry(int * unum, int * status);

extern int      DEBUGLEVEL;

int             gotalarm;

void 
gotalarm_sig(void)
{
	gotalarm = 1;
}

/*convert fsmb structure <-> smb_passwd.  direction 0 is passwd->VMS, 
anything else is VMS->passwd */

void convert_smb_fsmb(struct fsmb_type* fsmb_data, struct smb_passwd* pwd, int direction){
int usernamelen;
static pstring  user_name;
static unsigned char smbpwd[16];
static unsigned char smbntpwd[16];
char c;
char *p;
int uptr;
int looking;

  if(direction == 0){  /* passwd -> VMS */

     fsmb_data->acct_ctrl=pwd->acct_ctrl;  /* set the lower 16 bits of the acct_ctrl field */

     (void) strncpy(fsmb_data->username,"                                ",32);
     if(pwd->smb_name != NULL){ /*equivalent is all spaces*/
       usernamelen=strlen(pwd->smb_name);
       if(usernamelen >32)usernamelen=32;
       if(usernamelen > 0 ){
         (void) strncpy(&(fsmb_data->username[usernamelen-1]),pwd->smb_name,32-usernamelen);
       }
     }

     fsmb_data->uic = pwd->smb_userid;     

     if(pwd->smb_passwd == NULL){
	fsmb_data->acct_ctrl |= ACB_NOLANMANPWD; /* no lanman password defined */
        memset(fsmb_data->lanman_password,0,16);
     }
     else { /* there better be 16 chars here or BANG */
	memcpy(fsmb_data->lanman_password,pwd->smb_passwd,16*sizeof(char));
     }
     if(pwd->smb_nt_passwd == NULL){
	fsmb_data->acct_ctrl |= ACB_NOWNTPWD; /* no wnt password defined */
        memset(fsmb_data->wnt_password,0,16);
     }
     else {
	memcpy(fsmb_data->wnt_password,pwd->smb_nt_passwd,16*sizeof(char));
     }
  }
  else { /*  VMS -> passwd, this would be after reading from the password file */

    pwd->smb_passwd=NULL;
    pwd->smb_nt_passwd=NULL;
    pwd->acct_ctrl = 0x0000FFFF & fsmb_data->acct_ctrl;
    pwd->smb_userid=fsmb_data->uic;
    pwd->smb_name=&(user_name[0]);

    /* there are always 16 bytes in these, but they may not mean anything, 
       depending on what is stored in the top bits of the acct_ctrl fields  */

    memcpy(smbpwd,  fsmb_data->lanman_password,16*sizeof(char));
    memcpy(smbntpwd,fsmb_data->wnt_password,   16*sizeof(char));
    if( (fsmb_data->acct_ctrl & ACB_NOLANMANPWD) == 0)pwd->smb_passwd=&smbpwd[0];
    if( (fsmb_data->acct_ctrl & ACB_NOWNTPWD) == 0)pwd->smb_nt_passwd=&smbntpwd[0];

    /* construct contents of user_name */

    for(looking=1,uptr=0, p=&user_name[1]; uptr<=31 ; uptr++){
       c=fsmb_data->username[uptr];
       if(looking == 1){
          if(c != ' ' ){
            *p=c;
            p++;
            looking = 0;
          }
          else {
            *p=c;
	    p++;
	  }
       }
       *p='\0'; /* don't forget to terminate name */
    }
  }
}

/*************************************************************************
 Routine to search the smbpasswd file for an entry matching the username
 or user id.  if the name is NULL, then the smb_uid is used instead.
 *************************************************************************/
struct smb_passwd *get_smbpwd_entry(char *name, int smb_userid)
{
	/* Static buffers we will return. */
	static struct smb_passwd  pw_buf;
	long                 uidval;
	char                 *pfile = lp_smb_passwd_file();
	struct fsmb_type     fsmb_data;
	struct fsmb_type *  fsmb_passdata;


	if (!*pfile) {
	  DEBUG(0, ("No SMB password file set\n"));
	  return (NULL);
	}
	DEBUG(10, ("get_smbpwd_entry: opening file %s\n", pfile));

	fsmb_openfile(pfile, &fsmb_unum, &fsmb_status);
	
	if (fsmb_status != 0) {
	   DEBUG(0, ("get_smbpwd_entry: unable to open file %s\n", pfile));
	   return NULL;
	}

        /* store the data into the fsmb_data structure */

        convert_smb_fsmb(fsmb_passdata, (struct smb_passwd*) &pw_buf, 1);

	if (name != NULL)
	{

	  DEBUG(10, ("get_smbpwd_entry: search by name: %s\n", name));

	  /* name has already been packed into fsmb_data.username fields, 
             and that is being used as the key */

	  fsmb_find_by_name(&fsmb_unum, &(fsmb_data.username[1]), fsmb_passdata, &fsmb_status);
	  if(fsmb_status != 0){
	    DEBUG(0, ("SMB username not found\n"));
	    return (NULL);
	  }
	}
	else
	{
	  uidval=smb_userid;
	  DEBUG(10, ("get_smbpwd_entry: search by smb_userid: %x\n", smb_userid));
	  fsmb_find_by_uic(&fsmb_unum, &smb_userid, fsmb_passdata, &fsmb_status);
	  if(fsmb_status != 0){
	    DEBUG(0, ("SMB username not found\n"));
	    return (NULL);
	  }
	}

        /* retrieved the data, convert to pwd struct and return */

        convert_smb_fsmb(fsmb_passdata, (struct smb_passwd*) &pw_buf, 1);

	if (((pw_buf.acct_ctrl & ACB_DISABLED) | (pw_buf.acct_ctrl & ACB_NOLANMANPWD)) != 0){    /* Password deliberately invalid - end here. */
	   DEBUG(10, ("get_smbpwd_entry: entry invalidated for user %s\n", name));
	   fsmb_closefile(&fsmb_unum, &fsmb_status);
	   return NULL;
	}

	fsmb_closefile(&fsmb_unum, &fsmb_status);

	DEBUG(5, ("get_smbpwd_entry: returning passwd entry for user %s, uid %d\n",name, uidval));
	return &pw_buf;
}

/*
 * Add a new entry
 */

BOOL add_smbpwd_entry(struct smb_passwd* pwd)
{
	/* Static buffers we will return. */
	struct fsmb_type     fsmb_data;
	struct fsmb_type *  fsmb_passdata;

	char           *pfile = lp_smb_passwd_file();

	fsmb_openfile(pfile, &fsmb_unum, &fsmb_status);
	
	if (fsmb_status != 0) {
	   DEBUG(0, ("get_smbpwd_entry: unable to open file %s\n", pfile));
	   return False;
	}

        /* store the data into the fsmb_data structure */

        convert_smb_fsmb(fsmb_passdata, (struct smb_passwd*) pwd, 1);

        /* add it to the file (aren't indexed files wonderful!) */

	fsmb_append_entry(&fsmb_unum, fsmb_passdata, &fsmb_status);
	if(fsmb_status != 0){
	  DEBUG(0, ("Could not add new entry\n"));
	  fsmb_closefile(&fsmb_unum, &fsmb_status);
	  return False;
	}

	return True;
}

/*
 * Routine to search the smbpasswd file for an entry matching the username.
 * and then modify its password entry
 * override = 0, normal
 * override = 1, don't care about XXXXXXXX'd out password or NO PASS
 */

BOOL mod_smbpwd_entry(struct smb_passwd* pwd, BOOL override)
{
	char             *pfile = lp_smb_passwd_file();
	struct fsmb_type *  fsmb_passdata;
	BOOL found_entry = False;

	if (!*pfile){
	  DEBUG(0, ("No SMB password file set\n"));
	  return False;
	}
	DEBUG(10, ("mod_smbpwd_entry: opening file %s\n", pfile));

	fsmb_openfile(pfile, &fsmb_unum, &fsmb_status);
	
	if (fsmb_status != 0) {
	   DEBUG(0, ("mod_smbpwd_entry: unable to open file %s\n", pfile));
	   return False;
	}

        /* store the data into the fsmb_data structure */

        convert_smb_fsmb(fsmb_passdata, (struct smb_passwd*) pwd, 1);

        /* find the current record */

	fsmb_find_by_name(&fsmb_unum, fsmb_passdata->username, fsmb_passdata, &fsmb_status);
	if(fsmb_status != 0){
	  DEBUG(0, ("SMB username not found\n"));
	  fsmb_closefile(&fsmb_unum, &fsmb_status);
	  return False;
	}

	if(!override && ((fsmb_passdata->acct_ctrl & ACB_DISABLED) != 0)){
	  DEBUG(0, ("SMB username modification disallowed for disabled user\n"));
	  fsmb_closefile(&fsmb_unum, &fsmb_status);
	  return False;
	}

        /* store the data into the fsmb_data structure AGAIN, as last 
           operation trashed it*/

        convert_smb_fsmb(fsmb_passdata, (struct smb_passwd*) pwd, 1);

	fsmb_modify_last_entry(&fsmb_unum, fsmb_passdata, &fsmb_status);
	if(fsmb_status != 0){
	  DEBUG(0, ("SMB could not update existing record for a user\n"));
	  fsmb_closefile(&fsmb_unum, &fsmb_status);
	  return False;
	}

	fsmb_closefile(&fsmb_unum, &fsmb_status);
	return True;
}
